@@ -6,39 +6,29 @@
 from __future__ import division
 
 import sys
-if sys.version > '3':
-    xrange = range
-
-if sys.version < '3':
-    def b(x):
-        return x
-
-    def u(x):
-        return unicode(x)
-else:
-    def b(x):
-        return x.encode('ISO-8859-1')
-
-    def u(x):
-        return x
-
-
 import math
 import types
 import socket
 import os
+import sys
 import errno
 import time
 import struct
 import re
 import string
-from collections import deque
+from collections import deque, Iterable, Sequence
+from abc import ABCMeta
 
 try:
     import simplejson as json
 except ImportError:
     import json
 
+try:
+    import win32file
+except ImportError:
+    pass
+
 # Import ioflo libs
 from .globaling import *
 from .odicting import odict
@@ -46,8 +36,6 @@ from .odicting import odict
 from .consoling import getConsole
 console = getConsole()
 
-
-
 def metaclassify(metaclass):
     """
     Class decorator for creating a class with a metaclass.
@@ -83,6 +71,54 @@ def metaclassify(metaclass):
         return metaclass(cls.__name__, cls.__bases__, originals)
     return wrapper
 
+@metaclassify(ABCMeta)
+class NonStringIterable:
+    """ Allows isinstance check for iterable that is not a string
+    """
+    #__metaclass__ = ABCMeta
+
+    @classmethod
+    def __subclasshook__(cls, C):
+        if cls is NonStringIterable:
+            if (not issubclass(C, basestring) and issubclass(C, Iterable)):
+                return True
+        return NotImplemented
+
+@metaclassify(ABCMeta)
+class NonStringSequence:
+    """ Allows isinstance check for sequence that is not a string
+    """
+    #__metaclass__ = ABCMeta
+
+    @classmethod
+    def __subclasshook__(cls, C):
+        if cls is NonStringSequence:
+            if (not issubclass(C, basestring) and issubclass(C, Sequence)):
+                return True
+        return NotImplemented
+
+def nonStringIterable(obj):
+    """
+    Returns True if obj is non-string iterable, False otherwise
+
+    Future proof way that is compatible with both Python3 and Python2 to check
+    for non string iterables.
+    Assumes in Python3 that, basestring = (str, bytes)
+
+    Faster way that is less future proof
+    return (hasattr(x, '__iter__') and not isinstance(x, basestring))
+    """
+    return (not isinstance(obj, basestring) and isinstance(obj, Iterable))
+
+def nonStringSequence(obj):
+    """
+    Returns True if obj is non-string sequence, False otherwise
+
+    Future proof way that is compatible with both Python3 and Python2 to check
+    for non string sequences.
+    Assumes in Python3 that, basestring = (str, bytes)
+    """
+    return (not isinstance(obj, basestring) and isinstance(obj, Sequence) )
 
 class Fifo(deque):  #new-style class to add put get methods
     """ Extends deque to support more convenient FIFO queue access
@@ -624,11 +660,12 @@ class SocketUdpNb(object):
 
             return (data,sa)
         except socket.error as ex: # 2.6 socket.error is subclass of IOError
-            if ex.errno == errno.EAGAIN: #Resource temporarily unavailable on os x
+            # Some OSes define errno differently so check for both
+            if ex.errno == errno.EAGAIN or ex.errno == errno.EWOULDBLOCK:
                 return ('',None) #receive has nothing empty string for data
             else:
                 emsg = "socket.error = {0}: receiving at {1}\n".format(ex, self.ha)
-                console.terse(emsg)
+                console.profuse(emsg)
                 raise #re raise exception ex1
 
     def send(self, data, da):
@@ -642,7 +679,7 @@ class SocketUdpNb(object):
             result = self.ss.sendto(data, da) #result is number of bytes sent
         except socket.error as ex:
             emsg = "socket.error = {0}: sending from {1} to {2}\n".format(ex, self.ha, da)
-            console.terse(emsg)
+            console.profuse(emsg)
             result = 0
             raise
 
@@ -809,7 +846,7 @@ class SocketUxdNb(object):
                 return ('',None) #receive has nothing empty string for data
             else:
                 emsg = "socket.error = {0}: receiving at {1}\n".format(ex, self.ha)
-                console.terse(emsg)
+                console.profuse(emsg)
                 raise #re raise exception ex1
 
     def send(self, data, da):
@@ -823,7 +860,7 @@ class SocketUxdNb(object):
             result = self.ss.sendto(data, da) #result is number of bytes sent
         except socket.error as ex:
             emsg = "socket.error = {0}: sending from {1} to {2}\n".format(ex, self.ha, da)
-            console.terse(emsg)
+            console.profuse(emsg)
             result = 0
             raise
 
@@ -835,6 +872,175 @@ class SocketUxdNb(object):
 
         return result
 
+class WinMailslotNb(object):
+    """
+    Class to manage non-blocking reads and writes from a
+    Windows Mailslot
+
+    Opens a non-blocking mailslot
+    Use instance method to close socket
+
+    Needs Windows Python Extensions
+    """
+
+    def __init__(self, ha=None, bufsize=1024, path='', log=False):
+        """
+        Init method for instance
+
+        bufsize = default mailslot buffer size
+        path = path to directory where logfiles go.  Must end in /.
+        ha = basename for mailslot path.
+        """
+        self.ha = ha
+        self.bs = bufsize
+        self.ms = None   # Mailslot needs to be opened
+
+        self.path = path
+        self.txLog = None     # Transmit log
+        self.rxLog = None     # receive log
+        self.log = log
+
+    def open(self):
+        """
+        Opens mailslot in nonblocking mode
+        """
+        try:
+            self.ms = win32file.CreateMailslot(self.ha, 0, 0, None)
+            # ha = path to mailslot
+            # 0 = MaxMessageSize, 0 for unlimited
+            # 0 = ReadTimeout, 0 to not block
+            # None = SecurityAttributes, none for nothing special
+        except win32file.error as ex:
+            console.terse('mailslot.error = {0}'.format(ex))
+            return False
+
+        if self.log:
+            if not self.openLogs():
+                return False
+
+        return True
+
+    def reopen(self):
+        """
+        Clear the ms and reopen
+        """
+        self.close()
+        return self.open()
+
+    def close(self):
+        '''
+        Close the mailslot
+        '''
+        if self.ms:
+            win32file.CloseHandle(self.ms)
+            self.ms = None
+
+        self.closeLogs()
+
+    def receive(self):
+        """
+        Perform a non-blocking read on the mailslot
+
+        Returns tuple of form (data, sa)
+        if no data, returns ('', None)
+          but always returns a tuple with 2 elements
+
+        Note win32file.ReadFile returns a tuple: (errcode, data)
+
+        """
+        try:
+            data = win32file.ReadFile(self.ms, self.bs)
+
+            # Mailslots don't send their source address
+            # We can pick this up in a higher level of the stack if we
+            # need
+            sa = None
+
+            message = "Server at {0} received {1}\n".format(
+                self.ha, data[1])
+
+            console.profuse(message)
+
+            if self.log and self.rxLog:
+                self.rxLog.write("%s\n%s\n" % (sa, repr(data[1])))
+
+            return (data[1], sa)
+
+        except win32file.error:
+            return ('', None)
+
+    def send(self, data, destmailslot):
+        """
+        Perform a non-blocking write on the mailslot
+        data is string in python2 and bytes in python3
+        da is destination mailslot path
+        """
+
+        try:
+            f = win32file.CreateFile(destmailslot,
+                                     win32file.GENERIC_WRITE | win32file.GENERIC_READ,
+                                     win32file.FILE_SHARE_READ,
+                                     None, win32file.OPEN_ALWAYS, 0, None)
+        except win32file.error as ex:
+            emsg = 'mailslot.error = {0}: opening mailslot from {1} to {2}\n'.format(
+                ex, self.ha, destmailslot)
+            console.terse(emsg)
+            result = 0
+            raise
+
+        try:
+            result = win32file.WriteFile(f, data)
+            console.profuse("Server at {0} sent {1} bytes\n".format(self.ha,
+                                                                    result))
+        except win32file.error as ex:
+            emsg = 'mailslot.error = {0}: sending from {1} to {2}\n'.format(ex, self.ha, destmailslot)
+            console.terse(emsg)
+            result = 0
+            raise
+
+        finally:
+            win32file.CloseHandle(f)
+
+        if self.log and self.txLog:
+            self.txLog.write("%s %s bytes\n%s\n" %
+                             (str(destmailslot), str(result), repr(data)))
+
+        # WriteFile returns a tuple, we only care about the number of bytes
+        return result[1]
+
+    def openLogs(self, path = ''):
+        """Open log files
+
+        """
+        date = time.strftime('%Y%m%d_%H%M%S',time.gmtime(time.time()))
+        name = "%s%s_%s_%s_tx.txt" % (self.path, self.ha[0], str(self.ha[1]), date)
+        try:
+            self.txLog = open(name, 'w+')
+        except IOError:
+            self.txLog = None
+            self.log = False
+            return False
+        name = "%s%s_%s_%s_rx.txt" % (self.path, self.ha[0], str(self.ha[1]), date)
+        try:
+            self.rxLog = open(name, 'w+')
+        except IOError:
+            self.rxLog = None
+            self.log = False
+            return False
+
+        return True
+
+    def closeLogs(self):
+        """Close log files
+
+        """
+        if self.txLog and not self.txLog.closed:
+            self.txLog.close()
+        if self.rxLog and not self.rxLog.closed:
+            self.rxLog.close()
+
+
+
 #Utility Functions
 
 def TotalSeconds(td):
@@ -959,27 +1165,6 @@ def Just(n, seq):
 
 just = Just #alias
 
-from abc import ABCMeta,  abstractmethod
-@metaclassify(ABCMeta)
-class NonStringIterable:
-    """ Check for iterable that is not a string
-        Works in python 2.6 to 3.x with isinstance(x, NonStringIterable)
-
-    """
-    #__metaclass__ = ABCMeta
-
-    @abstractmethod
-    def __iter__(self):
-        while False:
-            yield None
-
-    @classmethod
-    def __subclasshook__(cls, C):
-        if cls is NonStringIterable:
-            if any("__iter__" in B.__dict__ for B in C.__mro__):
-                return True
-        return NotImplemented
-
 # Faster to use precompiled versions in globaling
 def IsPath(s):
     """Returns True if string s is valid Store path name
@@ -2115,4 +2300,3 @@ def LoadJson(filename = ""):
         return it
 
 loadJson = LoadJson
-
