@@ -6,7 +6,8 @@ import sys
 if sys.version > '3':
     xrange = range
 import copy
-from collections import deque
+from collections import deque, Mapping
+import uuid
 
 from .odicting import odict
 from .globaling import *
@@ -38,6 +39,7 @@ class Framer(tasking.Tasker):
        instance attributes
             .main = main frame when this framer is an auxiliary
             .original = clone state, False if clone True if not clone
+            .insular = clone that is visible only to main framer
             .done = auxiliary completion state True or False when an auxiliary
             .elapsed = elapsed time from outline change
             .elapsedShr = share where .elapsed is stored for logging and need checks
@@ -59,6 +61,8 @@ class Framer(tasking.Tasker):
 
             .frameNames = frame name registry , name space of frame names
             .frameCounter = frame name registry counter
+
+            .moots = list of moot framers to be cloned
     """
     #Counter = 0
     #Names = {}
@@ -72,6 +76,7 @@ class Framer(tasking.Tasker):
 
         self.main = None  #when aux framer, frame that is running this aux
         self.original = True  # as in not a clone
+        self.insular = False  # as in a clone that is visible only to the main framer
         self.done = True #when aux or slave framer, completion state, set to False on enterAll
 
         self.stamp = 0.0 #beginning time to compute elapsed time since last outline change
@@ -100,6 +105,8 @@ class Framer(tasking.Tasker):
         self.frameNames = odict() #frame name registry for framer. name space of frame names
         self.frameCounter = 0 #frame name registry counter for framer
 
+        self.moots = []  # moot framers to be cloned
+
     def clone(self, name, period=0.0, schedule=AUX):
         """ Return clone of self named name
 
@@ -118,10 +125,11 @@ class Framer(tasking.Tasker):
                        store=self.store,
                        period=period,
                        schedule=schedule)
-        console.profuse("     Cloning Framer original '{0}' to clone '{1}'\n"
+        console.terse("         Cloning contents of Framer original '{0}' to clone '{1}'\n"
                         "".format(self.name, clone.name))
         clone.schedule = schedule
         clone.first = self.first # resolve later
+        clone.moots = copy.deepcopy(self.moots)
 
         clone.assignFrameRegistry() #Frame.names is cloned framer registry
         for frame in self.frameNames.values():
@@ -129,6 +137,104 @@ class Framer(tasking.Tasker):
 
         return clone
 
+    def resolve(self):
+        """Convert all the name strings for links to references to instance
+           by that name
+        """
+        console.terse("     Resolving Framer {0}\n".format(self.name))
+        self.resolveMoots()
+
+        self.assignFrameRegistry() #needed by act links below
+
+        #Resolve first frame link
+        if self.first:
+            self.first = resolveFrame(self.first, who=self.name, desc='first')
+        else:
+            raise excepting.ResolveError("No first frame link", self.name, self.first)
+
+        for frame in Frame.Names.values(): #all frames in this framer's name space
+            frame.resolve()
+
+        self.traceOutlines()
+
+        self.resolved = True
+
+    def resolveMoots(self):
+        """
+        Resolves .moots by cloning as appropriate.
+        .moots is list of dicts of form
+        {
+            original: framername,
+            clone: string,
+            schedule: constant,
+            human: string,
+            count: number
+        }
+
+        creates clone and adds to taskables if named clone
+        resolution looks up name string in appropriate registry and replaces
+        name string with link to object
+
+        Assumes Framer.names is .house's registry
+        """
+        #self.store.house.assignRegistries() # ensure Framer.names is houses registry
+        console.terse("       Resolving original moots for named clones ...\n")
+        for data in self.moots:
+            original = data['original']  # original name
+            clone = data['clone']  # clone name
+            schedule = data['schedule']
+            human = data['human']
+            count = data['count']
+            console.terse("         Cloning original '{0}' as named clone '{1}'\n"
+                            "".format(original, clone))
+            if clone == 'mine':  # invalid name for named clone
+                raise excepting.ResolveError("Invalid named clone name of 'mine'",
+                                             name=original,
+                                             value=self.name,
+                                             human=human,
+                                             count=count )
+
+            original = resolveFramer(original,
+                                     who=self.name,
+                                     desc='original',
+                                     contexts=[MOOT],
+                                     human=human,
+                                     count=count)
+            clone = original.clone(name=clone, schedule=schedule)
+            clone.original = False  # main frame will be fixed
+            self.store.house.resolvables.append(clone)
+            self.store.house.taskers.append(clone)
+            self.store.house.framers.append(clone)
+            if schedule == AUX:
+                self.store.house.auxes.append(clone)
+
+        self.moots = []
+
+    @staticmethod
+    def nameUid(prefix='clone', size=8):
+        '''
+        Returns unique name for Framer composed of prefix and random bytes
+        prefix is prefix to name
+        size is number of random bytes
+
+        Uses uuid.uuid4
+        '''
+        size = int(max(size, 1))
+        return ("{0}_{1}".format(prefix, uuid.uuid4().hex[:size]))
+
+    def traceOutlines(self):
+        """Trace and assign outlines for each frame in framer
+        """
+        console.terse("       Tracing outlines for framer {0}\n".format(self.name))
+
+        self.assignFrameRegistry()
+
+        for frame in Frame.Names.values(): #all frames in this framer's name space
+            frame.traceOutline()
+            frame.traceHead()
+            frame.traceHuman()
+            frame.traceHeadHuman()
+
     def assignFrameRegistry(self):
         """Point Frame class name registry dict and counter to .frameNames
            and .frameCounter.
@@ -146,7 +252,6 @@ class Framer(tasking.Tasker):
         self.elapsed = 0.0
         self.updateElapsed()
 
-
     def updateTimer(self):
         """update the elapsed time of framer in  current outline
            use store.stamp for current time reference
@@ -190,36 +295,6 @@ class Framer(tasking.Tasker):
             self.recurredShr.name, self.recurredShr.value, self.recurred))
         self.recurredShr.update(value = self.recurred)
 
-    def resolve(self):
-        """Convert all the name strings for links to references to instance
-           by that name
-        """
-        console.terse("     Resolving framer {0}\n".format(self.name))
-
-        self.assignFrameRegistry() #needed by act links below
-
-        for frame in Frame.Names.values(): #all frames in this framer's name space
-            frame.resolve()
-
-        #Resolve first frame link
-        if self.first:
-            self.first = resolveFrame(self.first, who=self.name, desc='first')
-        else:
-            raise excepting.ResolveError("No first frame link", self.name, self.first)
-
-    def traceOutlines(self):
-        """Trace and assign outlines for each frame in framer
-        """
-        console.terse("     Tracing outlines for framer {0}\n".format(self.name))
-
-        self.assignFrameRegistry()
-
-        for frame in Frame.Names.values(): #all frames in this framer's name space
-            frame.traceOutline()
-            frame.traceHead()
-            frame.traceHuman()
-            frame.traceHeadHuman()
-
     def change(self, actives, human = ''):
         """set .actives and .human to new outline actives
            and human readable version human
@@ -702,8 +777,8 @@ class Frame(registering.StoriedRegistry):
         """
         clone = Frame(name=self.name,
                       store=self.store,
-                      framer=framer)
-        console.profuse("       Cloning Frame '{0}' into Framer '{1}'\n".format(
+                      framer=framer.name) # only name so resolve framer later
+        console.terse("           Cloning Frame '{0}' into Framer '{1}'\n".format(
                                clone.name, framer.name))
 
         for aux in self.auxes:
@@ -747,6 +822,164 @@ class Frame(registering.StoriedRegistry):
 
         return clone
 
+    def resolve(self):
+        """Resolve links where links are instance name strings assigned during building
+           need to be converted to object references using instance name registry
+
+        """
+        console.concise("       Resolving Frame {0}\n".format(self.name))
+
+        self.resolveFramerLink()
+
+        self.resolveNextLink()
+        self.resolveOverLinks()
+        self.resolveUnderLinks()
+
+        self.resolveAuxLinks()
+
+        for act in self.beacts:
+            act.resolve()
+
+        for act in self.enacts:
+            act.resolve()
+
+        for act in self.reacts:
+            act.resolve()
+
+        for act in self.preacts:
+            act.resolve()
+
+        for act in self.exacts:
+            act.resolve()
+
+        for act in self.rexacts:
+            act.resolve()
+
+        for act in self.renacts:
+            act.resolve()
+
+    def resolveFramerLink(self):
+        """Resolve framer link """
+        if self.framer:
+            self.framer = framer = resolveFramer(self.framer,
+                                                who=self.name,
+                                                desc="frame's",)
+
+    def resolveNextLink(self):
+        """Resolve next link
+
+        """
+        if self.next_:
+            self.next_ = resolveFrame(self.next_, who=self.name, desc='next')
+
+    def resolveOverLinks(self):
+        """Starting with self.over climb over links resolving the links as needed along the way
+
+        """
+        over = self.over
+        under = self
+
+        while over: #not beyond top
+            if not isinstance(over, Frame): #over is name of frame not ref so resolve
+                name = over #make copy for later
+                try:
+                    over = Frame.Names[name] #get reference from Frame name registry
+                except KeyError:
+                    raise excepting.ResolveError("Bad over link in outline", self.name, name)
+
+                if over == self: #check for loop
+                    raise excepting.ResolveError("Outline overs create loop", self.name, under.name)
+
+                #attach under to over
+                if under.name in over.unders: #under name in unders as a result of script under cmd
+                    index = over.unders.index(under.name) #index = position in list
+                    over.unders[index] = under #replace under at position index
+                else: #otherwise append
+                    over.unders.append(under) #add to unders
+
+                #maybe should error check for duplicates in unders here
+
+                under.over = over #assign valid over ref
+
+            else: #over is valid frame reference so don't need to resolve
+                if over == self: #check for loop
+                    raise excepting.ResolveError("Outline overs create loop", self.name, under.name)
+
+            under = over
+            over = over.over #rise one level
+
+    def resolveUnderLinks(self):
+        """ Resolve under links """
+        for i, under in enumerate(self.unders):
+            self.unders[i] = resolveFrame(under, who=self.name, desc='under')
+
+        if len(set(self.unders)) != len(self.unders): # duplicates
+            raise excepting.ResolveError("Duplicate under", name=self.name, value=self.unders)
+
+    def resolveAuxLinks(self):
+        """ Resolve aux links
+
+            If aux.original
+               aux.main for each aux is not assigned here but is assigned when
+              frame.enter before aux.enterAll() so can reuse aux in other frames.
+            Otherwise
+               assign aux.main to self
+        """
+        for i, aux in enumerate(self.auxes):
+            if isinstance(aux, Mapping):
+                clone = aux['clone']
+                original = aux['original']
+                schedule = aux['schedule']
+                human = aux['human']
+                count = aux['count']
+                if schedule != AUX:
+                    msg = ("ResolveError: Invalid insular clone schedule '{0}' "
+                           "for {1}.".format(schedule, original))
+                    raise excepting.ResolveError(msg,
+                                                 name=clone,
+                                                 value=self.name,
+                                                 human=human,
+                                                 count=count)
+                if clone != 'mine':
+                    msg = "Aux insular clone name must be 'mine' not '{0}'".format(clone)
+                    raise excepting.ResolveError(msg,
+                                                 name=clone,
+                                                 value=self.name,
+                                                 human=human,
+                                                 count=count)
+
+                clone = Framer.nameUid(prefix=original)
+                while (clone in Framer.Names): # ensure unique
+                    clone = Framer.nameUid(prefix=original)
+
+                console.terse("         Cloning original '{0}' as insular clone '{1}'\n"
+                                            "".format(original, clone))
+                original = resolveFramer(original,
+                                         who=self.name,
+                                         desc='original',
+                                         contexts=[MOOT],
+                                         human=human,
+                                         count=count)
+                clone = original.clone(name=clone, schedule=schedule)
+                clone.original = False  # main frame will be fixed
+                clone.insular =  True #  local to this framer
+                self.auxes[i] = aux = clone
+                self.store.house.resolvables.append(clone)
+                self.framer.assignFrameRegistry()  # restore
+            else:
+                self.auxes[i] = aux = resolveFramer(aux,
+                                                who=self.name,
+                                                desc='aux',
+                                                contexts=[AUX])
+
+            if not aux.original: # clones get fixed main never released
+                if aux.main: # raise exception if aux.main is not None
+                    msg = "Aux already assigned to main '{0}'".format(aux.main.name)
+                    raise excepting.ResolveError(msg,
+                                                 name=aux.name,
+                                                 value=self.name)
+                aux.main = self
+
     def expose(self):
         """Prints out instance variables.
 
@@ -821,120 +1054,6 @@ class Frame(registering.StoriedRegistry):
                 frame = frame.over #keep going up outline
         return False #no loop
 
-    def resolveNextLink(self):
-        """Resolve next link
-
-        """
-        if self.next_:
-            self.next_ = resolveFrame(self.next_, who=self.name, desc='next')
-
-    def resolveOverLinks(self):
-        """Starting with self.over climb over links resolving the links as needed along the way
-
-        """
-        over = self.over
-        under = self
-
-        while over: #not beyond top
-            if not isinstance(over, Frame): #over is name of frame not ref so resolve
-                name = over #make copy for later
-                try:
-                    over = Frame.Names[name] #get reference from Frame name registry
-                except KeyError:
-                    raise excepting.ResolveError("Bad over link in outline", self.name, name)
-
-                if over == self: #check for loop
-                    raise excepting.ResolveError("Outline overs create loop", self.name, under.name)
-
-                #attach under to over
-                if under.name in over.unders: #under name in unders as a result of script under cmd
-                    index = over.unders.index(under.name) #index = position in list
-                    over.unders[index] = under #replace under at position index
-                else: #otherwise append
-                    over.unders.append(under) #add to unders
-
-                #maybe should error check for duplicates in unders here
-
-                under.over = over #assign valid over ref
-
-            else: #over is valid frame reference so don't need to resolve
-                if over == self: #check for loop
-                    raise excepting.ResolveError("Outline overs create loop", self.name, under.name)
-
-            under = over
-            over = over.over #rise one level
-
-    def resolveUnderLinks(self):
-        """ Resolve under links """
-        for i, under in enumerate(self.unders):
-            self.unders[i] = resolveFrame(under, who=self.name, desc='under')
-
-        if len(set(self.unders)) != len(self.unders): # duplicates
-            raise excepting.ResolveError("Duplicate under", name=self.name, value=self.unders)
-
-    def resolveAuxLinks(self):
-        """ Resolve aux links
-
-            If aux.original
-               aux.main for each aux is not assigned here but is assigned when
-              frame.enter before aux.enterAll() so can reuse aux in other frames.
-            Otherwise
-               assign aux.main to self
-        """
-        for i, aux in enumerate(self.auxes):
-            self.auxes[i] = aux = resolveFramer(aux,
-                                                who=self.name,
-                                                desc='aux',
-                                                contexts=[AUX])
-            if not aux.original: # clones get fixed main never released
-                aux.main = self
-
-    def resolveFramerLink(self):
-        """Resolve framer link """
-        if self.framer:
-            self.framer = framer = resolveFramer(self.framer,
-                                                who=self.name,
-                                                desc='framer',)
-
-    def resolve(self):
-        """Resolve links where links are instance name strings assigned during building
-           need to be converted to object references using instance name registry
-
-        """
-        console.profuse("Resolving frame {0} in framer {1}\n".format(
-            self.name, self.framer.name))
-
-        self.resolveFramerLink()
-
-        self.resolveNextLink()
-        self.resolveOverLinks()
-        self.resolveUnderLinks()
-
-        self.resolveAuxLinks()
-
-        for act in self.beacts:
-            act.resolve()
-
-        for act in self.enacts:
-            act.resolve()
-
-        for act in self.reacts:
-            act.resolve()
-
-        for act in self.preacts:
-            act.resolve()
-
-        for act in self.exacts:
-            act.resolve()
-
-        for act in self.rexacts:
-            act.resolve()
-
-        for act in self.renacts:
-            act.resolve()
-
-
-
     def findBottom(self):
         """Finds the bottom most frame for outline that this frame lives in
 
@@ -1052,8 +1171,8 @@ class Frame(registering.StoriedRegistry):
             #if aux.main is not None then it has not been released and so
             #we can't enter unless its ourself for forced re-entry
             if aux.main and (aux.main is not self): # does aux belong to another frame
-                console.profuse("    False. Invalid aux {0} in use "
-                        "by another frame {1}".format(aux.name, aux.main.name))
+                console.concise("    False. Invalid aux '{0}' in use "
+                        "by another frame '{1}'\n".format(aux.name, aux.main.name))
                 return False
 
             if not aux.checkStart(): #performs entry checks beacts
@@ -1269,7 +1388,8 @@ def resolveFramer(framer, who='', desc='framer', contexts=None,
                                          who,
                                          human,
                                          count)
-        console.terse("    Resolved {0} as {1} in {2}".format(desc, framer.name, who))
+        console.concise("         Resolved {0} Framer '{1}' in {2}\n"
+                      "".format(desc, framer.name, who))
     return framer
 
 ResolveFramer = resolveFramer
@@ -1282,14 +1402,37 @@ def resolveFrame(frame, who='', desc='act', human='', count=None):
     """
     if not isinstance(frame, Frame): # not instance so name
         if frame not in Frame.Names:
-            raise excepting.ResolveError("ResolveError: Bad {0} frame link name".format(desc),
+            raise excepting.ResolveError("ResolveError: Bad {0} Frame link name".format(desc),
                                          frame,
                                          who,
                                          human,
                                          count)
         frame = Frame.Names[frame] #replace frame name with frame
+        console.concise("         Resolved {0} Frame '{1}' in {2}\n"
+                              "".format(desc, frame.name, who))
 
     return frame
 
 ResolveFrame = resolveFrame
 
+def resolveFrameOfFramer(frame, framer, who='', desc='act', human='', count=None):
+    """ Returns resolved frame instance from frame
+        frame may be name of frame or instance
+
+        Resolves relative to the framer's .frameNames registry
+    """
+    if not isinstance(frame, Frame): # not instance so name
+        if frame not in framer.frameNames:
+            raise excepting.ResolveError("ResolveError: Bad {0} Frame link"
+                        " name in Framer '{1}'".format(desc, framer.name),
+                                         frame,
+                                         who,
+                                         human,
+                                         count)
+        frame = framer.frameNames[frame] #replace frame name with frame
+        console.concise("         Resolved {0} Frame '{1}' in {2}\n"
+                              "".format(desc, frame.name, who))
+
+    return frame
+
+ResolveFrameOfFramer = resolveFrameOfFramer
